    processor  6502
    include    vcs.h
    include    macro.h

; Compile switches

NTSC                    = 0
PAL                     = 1
COMPILE_VERSION         = NTSC

MAXOBJECTS      = $09   ;
LINES           = $40
TOPDIFF         = 34
MIDDLEDIFF      = 18
LINE2SLEEP      = 17
LINE1SLEEP      = 10
RIGHTBORDER     = 141
CROSSPACE       = LINES-6
CROSSYMAX       = 58
CROSSXMIN       = 11
CROSSXMAX       = 148

BROWN       = $FA
RED         = $44
BLACK       = $00
BLUE        = $98
WHITE       = $0F
YELLOW      = $1E
GREEN       = $CA
GREY        = $04

    SEG.U vars
    ORG $80

; Both Banks:
waveCounter     ds 1                ; desired/current wave
frameCounter    ds 1                ; Incremented every frame
rnd             ds 1                ; Random #
tempVar1        ds 1
tempVar2        ds 1
tempVar3        ds 1
tempVar4        ds 1

; This Bank:
xpos            ds #MAXOBJECTS      ; horizontal position of sprites
ypos            ds #MAXOBJECTS      ; vertical position of sprites
xymov           ds #MAXOBJECTS      ; horizontal motion for sprite
zmov            ds #MAXOBJECTS      ; horizontal motion for sprite
sprtsizetype    ds #MAXOBJECTS      ; list of sprite colors
vline           ds 1                ; current vertical line being drawn
p0count         ds 1                ; number of lines left to draw for player 0
c7shapeptr      ds 2                ;
laserCounter    ds 1                ; Decremented every frame
expCounter      ds 1                ; Decremented every frame
shotCounter     ds 1                ; Decremented every frame
shotcountinit   ds 1                ; Start value for shotCounter
shotspeedinit   ds 1                ; Start value for shot speed
specialon       ds 1                ; 00: ready to launch FF: already on
shipCounter     ds 1                ; Current # of ships
bVoidHandled    ds 1                ; Flag if void obj was handled
waveTimer       ds 1                ; indicating the length of the wave
yposBackup      ds 3                ; Starfield info for kernel
tempPtr         ds 2                ; temp pointer

rocketPtr1      ds 2
domePtr1        ds 2
towersPtr1      ds 2
towersPtr2      ds 2
smartiePtr1     ds 2
smartiePtr2     ds 2
smartiePtr3     ds 2
basePtr1        ds 2
cannonPtr1      ds 2
cannonPtr2      ds 2
crossoffset     ds 1
crossypos       ds 1
crossxpos       ds 1

charPtr         ds 12
SHIELDPFT       ds 6
SHIELDPFB       ds 6
bcdScore        ds 3

    SEG     Bank0
    ORG     $F000

MainScreen
;-----------------------------------------------------------
;SPACE DISPLAY KERNEL TOP
;-----------------------------------------------------------

    STA WSYNC
    STA HMOVE
    STY VBLANK          ; End vertical blanking

    JSR DrawScore

    STA WSYNC
    STA WSYNC

    LDX #$00
    LDA crossxpos
    JSR PosElement
    STA WSYNC

    LDA #BLUE
    STA COLUP0
    LDA #$00
    STA p0count
    LDA #LINES          ; number of lines to draw on screen
    STA vline
    LDA #%10000000
    STA tempVar1    ; Init p0count
    LDA #MAXOBJECTS-1   ; all objects
    LDA #$08            ; reflect spaceships
    STA REFP1

;-----------------------------------------------------------
;SPACE DISPLAY KERNEL
;-----------------------------------------------------------

NextLine
    STA WSYNC           ; Wait for horizontal sync
    STA HMOVE

; Draw crosshair
    LDY crossoffset
    LDX crosshair,Y
    STX GRP0

; Draw starfield
    LDY vline
    CPY yposBackup+2
    PHP
    CPY yposBackup+1
    PHP
    CPY yposBackup
    PHP

; Reset stack to RESBL
    LDX #$1F
    TXS
    TAX
    INC crossoffset

; To reposition or not to reposition?
    LDA tempVar1        ; Are we to reposition a sprite?
    BMI Notstart        ; N: Skip first kernel part
    DEC tempVar1        ; Clear Draw Flag

; Set new ships size
    LDY sprtsizetype,X
    LDA shapeheight,Y
    STA p0count         ; Set object size

; Prepare Positioning ship
    LDA xpos,X      ; A = desired X-position!
    STA WSYNC
.wait
    SBC #$0F        ; Battlezone calculate/wait
    BCS .wait       ; 2 in 1 magic !
    AND #$0F        ; Manuels "forget about EOR/ASL
    TAY             ; nonsense" magic :-)
    LDA hmovetab,Y  ;
    STA HMP1        ; Fine Position sprite 1
    DEC vline
    TXA
    STA RESP1       ;
    JMP NextLine    ; Immediate WSYNC/HMOVE follows!

SkipDraw
    LDA shapeclrtab,Y   ; Get sprite color
    STA COLUP1
    LDA #$00
    BEQ NotDraw

Notstart
    STA HMCLR
    LDY sprtsizetype,X
    LDA shapeptrtab,Y
    STA c7shapeptr     ; Set space object pointer

    STA WSYNC
    DEC p0count
    BMI SkipDraw
    LDY p0count
    LDA (c7shapeptr),y
NotDraw
    STA GRP1

; Ok, the miraculous skipping code
; I'm trying to comment that as good as possible... :-)

    LDY vline
    CPY #$02            ; Y: Are we already to close to the bottom?
    BCC Continue        ; Y: -> Bail out
    LDA xpos,X          ; Next sprite invisible?
    CMP #RIGHTBORDER
    BCS ContinueInv     ; -> Skip to next sprite
    LDA p0count         ; If we are already drawing a sprite
    BPL Continue        ; -> bail out completely
    TYA
    CMP ypos,X          ; Check if sprite starts now
    BNE CheckDoable     ; N: Check if we already passed it's line
    LDA #$00            ;
    STA tempVar1        ; N: Set reposition flag
    BEQ Continue        ; Bail out

CheckDoable
    BPL Continue        ; We've to wait some more for this sprite...

Continue2
    DEX                 ; Oh, we're too late for this one, try next
    BPL Continue        ;

Continue3
    LDX #$00            ; We should never have an index < 0

Continue
    TXA
    DEY
    STY vline
    BMI BottomScape
    JMP NextLine

; We eventually manage to skip away two invisible sprites in a row
ContinueInv
    DEX                 ; Oh, we're too late for this one, try next
    LDA xpos,X          ; Next sprite invisible?
    CMP #RIGHTBORDER
    BCS Continue2       ; -> Skip to next sprite
    TXA
    BMI Continue3
    BPL Continue

BottomScape
    LDY #47
    LDX #$1F
    TXS
    LDA #$00
    CPY yposBackup+2
    STA WSYNC
    PHP
    CPY yposBackup+1
    PHP
    CPY yposBackup
    PHP
    STA GRP0
    STA CTRLPF          ; Refleceted high priority PF
    STA WSYNC
    STA GRP1
    DEY
    DEY

    LDX #$1F
    TXS
    CPY yposBackup+2
    STA WSYNC
    STA.w RESP1
    PHP
    CPY yposBackup+1
    PHP
    CPY yposBackup
    PHP
    STA RESP0
    LDA #$30
    STA HMP1
    LDA #$B0
    STA HMP0
    DEY
    LDA #$80
    STA HMM0
    STA HMBL
    STA HMM1
    LDA frameCounter
    STA COLUPF
    SLEEP 20
    STA HMOVE
    DEY

NextTopLine
    LDX #$1F
    TXS
    LAX (cannonPtr1),Y
    LDA CANNONCOL-TOPDIFF,Y
    CPY yposBackup+2
    STA WSYNC
    PHP
    CPY yposBackup+1
    PHP
    CPY yposBackup
    PHP
    STX GRP0
    STA COLUP0
    STA COLUP1
    LDA (cannonPtr2),Y
    STA GRP1
    DEY

    STA WSYNC
    LDA (cannonPtr1),Y
    STA GRP0
    LDA (cannonPtr2),Y
    STA GRP1
    DEY
    CPY #34
    BCS NextTopLine

    LDA #RED
    LDX #$1F
    TXS
    LDX SHIELDPFT+0
    STA WSYNC
    STA COLUPF
    CPY yposBackup+2
    PHP
    CPY yposBackup+1
    PHP
    STX PF0
    LDA SHIELDPFT+1
    STA PF1
    LDA SHIELDPFT+2
    STA PF2
    LDA SHIELDPFT+5
    STA PF0
    LDA SHIELDPFT+4
    STA PF1
    LDA SHIELDPFT+3
    STA PF2
    DEY

    STA WSYNC
    LDA SHIELDPFT+0
    STA PF0
    LDA SHIELDPFT+1
    STA PF1
    LDA SHIELDPFT+2
    STA PF2
    LDX #$1F
    TXS
    SLEEP 8
    LDA SHIELDPFT+5
    STA PF0
    LDA SHIELDPFT+4
    STA PF1
    LDA SHIELDPFT+3
    STA PF2
    DEY

    STA WSYNC
    LDX SHIELDPFT+0
    CPY yposBackup+2
    PHP
    CPY yposBackup+1
    PHP
    STX PF0
    LDA SHIELDPFT+1
    STA PF1
    LDA SHIELDPFT+2
    STA PF2
    LDA SHIELDPFT+5
    STA PF0
    LDA SHIELDPFT+4
    STA PF1
    LDA SHIELDPFT+3
    STA PF2
    DEY

    STA WSYNC
    LDA #YELLOW
    STA COLUPF
    LDA SHIELDPFB+0
    STA PF0
    LDA SHIELDPFB+1
    STA PF1
    LDA SHIELDPFB+2
    STA PF2
    LDX #$1F
    TXS
    SLEEP 3
    LDA SHIELDPFB+5
    STA PF0
    LDA SHIELDPFB+4
    STA PF1
    LDA SHIELDPFB+3
    STA PF2
    DEY

    STA WSYNC
    LDX SHIELDPFB+0
    CPY yposBackup+2
    PHP
    CPY yposBackup+1
    PHP
    STX PF0
    LDA SHIELDPFB+1
    STA PF1
    LDA SHIELDPFB+2
    STA PF2
    LDA SHIELDPFB+5
    STA PF0
    LDA SHIELDPFB+4
    STA PF1
    LDA SHIELDPFB+3
    STA PF2
    DEY

    STA WSYNC
    LDA SHIELDPFB+0
    STA PF0
    LDA SHIELDPFB+1
    STA PF1
    LDA SHIELDPFB+2
    STA PF2
    LDX #$1F
    TXS
    SLEEP 8
    LDA SHIELDPFB+5
    STA PF0
    LDA SHIELDPFB+4
    STA PF1
    LDA SHIELDPFB+3
    STA PF2
    DEY

    SLEEP 13

    LDX #BROWN
    LDA #$00
    STA PF1
    STA PF2
    CPY yposBackup+2
    LDA #$05
    STA CTRLPF          ; Refleceted high priority PF
    PHP
    CPY yposBackup+1
    PHP
    STX COLUPF
    LDA #%00110000
    STA PF0
    LDA #$00
    STA GRP0
    STA GRP1
    STA RESP0
    LDA #$01
    STA HMP0
    LDA #$D0
    STA HMP1
    STA RESP1
    DEY
    LDA #%101
    STA NUSIZ1
    LDA #%110
    STA NUSIZ0
    SLEEP 13
    STA HMOVE
    LDA BOTTOMPF0,Y
    STA PF0
    LDA BOTTOMPF1,Y
    STA PF1
    LDA BOTTOMPF2,Y
    STA PF2
    DEY

NextMiddleLine
    LDX #$1F
    TXS
    LDX SMARTIECOL-MIDDLEDIFF,Y
    CPY yposBackup+2
    STA WSYNC
    PHP
    CPY yposBackup+1
    PHP
    LDA (smartiePtr1),Y
    STA GRP0
    STX COLUP0
    LDA (basePtr1),Y
    STA GRP1
    LDA (smartiePtr2),Y
    STA GRP0
    LDA BASECOL-MIDDLEDIFF,Y
    STA COLUP1
    LDA (smartiePtr3),Y
    STA GRP0
    DEY

    LDX SMARTIECOL-MIDDLEDIFF,Y
    LDA BASECOL-MIDDLEDIFF,Y
    STA COLUP1
    LDA BOTTOMPF0,Y
    STA WSYNC
    STA PF0
    LDA BOTTOMPF1,Y
    STA PF1
    LDA (smartiePtr1),Y
    STA GRP0
    STX COLUP0
    LDA BOTTOMPF2,Y
    STA PF2
    LDA (smartiePtr2),Y
    STA GRP0
    LDA (basePtr1),Y
    STA GRP1
    LDA (smartiePtr3),Y
    STA GRP0
    DEY
    CPY #19
    BCS NextMiddleLine

    STA WSYNC
    LDX #$1F
    TXS
    CPY yposBackup+2
    PHP
    CPY yposBackup+1
    PHP
    LDA #%110
    STA NUSIZ0
    SLEEP 10
    STA RESP1
    DEY
    SLEEP 18
    STA RESP0
    STA WSYNC
    LDA BOTTOMPF0,Y
    STA PF0
    LDA BOTTOMPF1,Y
    STA PF1
    LDA BOTTOMPF2,Y
    STA PF2
    DEY

    SLEEP 40

NextBottomLine
    STA GRP0
    LDX #$1F
    TXS
    LDX ROCKETCOL,Y
    CPY yposBackup+2
    PHP
    CPY yposBackup+1
    PHP
    LDA (rocketPtr1),Y
    STA GRP0
    STX COLUP0
    LDA (domePtr1),Y
    STA GRP1
    LDA DOMECOL,Y
    STA COLUP1
    LDA (towersPtr1),Y
    STA GRP0
    LDA TOWERSCOL,Y
    STA COLUP0
    LDA (towersPtr2),Y
    DEY
    LDX ROCKETCOL,Y
    STA GRP0
    LDA DOMECOL,Y
    STA COLUP1
    LDA BOTTOMPF0,Y
    STA.w PF0
    LDA BOTTOMPF1,Y
    STA PF1
    LDA (rocketPtr1),Y
    STA GRP0
    STX COLUP0
    LDA (domePtr1),Y
    STA GRP1
    LDA BOTTOMPF2,Y
    STA PF2
    LDA (towersPtr1),Y
    STA GRP0
    LDA TOWERSCOL,Y
    STA COLUP0
    LDA (towersPtr2),Y
    DEY
    BPL NextBottomLine

    LDX #$FF
    TXS

    STA WSYNC
    STA WSYNC
    LDA #$00
    STA PF0
    STA PF1
    STA PF2
    STA ENABL
    STA ENAM1
    STA NUSIZ1
    STA NUSIZ0

    LDA #$02            ;
    STA VBLANK          ;   start vertical blanking

    IF COMPILE_VERSION = PAL
        LDA #$41            ;
    ELSE
        LDA #$23
    ENDIF

    STA TIM64T          ; 30 lines overscan

    JSR Joy
    JMP OverscanLoop

;-----------------------------------------------------------
; CARTRIDGE MAIN ENTRY & INIT
;-----------------------------------------------------------

Start
    CLEAN_START

    SET_POINTER rocketPtr1, ROCKET
    SET_POINTER domePtr1, DOME
    SET_POINTER towersPtr1, TOWERS
    SET_POINTER towersPtr2, TOWERS

    SET_POINTER smartiePtr1, SMARTIER-MIDDLEDIFF
    SET_POINTER smartiePtr2, SMARTIER-MIDDLEDIFF
    SET_POINTER smartiePtr3, SMARTIEL-MIDDLEDIFF
    SET_POINTER basePtr1,    BASE-MIDDLEDIFF

    SET_POINTER cannonPtr1, CANON-TOPDIFF
    SET_POINTER cannonPtr2, CANON-TOPDIFF

    LDX #11
InitShield
    JSR Random
    JSR Random
    STA SHIELDPFT,X
    DEX
    BPL InitShield

    LDA #CROSSYMAX/2
    STA crossypos
    LDA #(CROSSXMAX-CROSSXMIN)/2+CROSSXMIN
    STA crossxpos

    LDA #BROWN
    STA COLUPF

    LDA #$0A
    STA waveTimer

    LDA #%00100000
    STA laserCounter

; Init correct GFX page
    LDX #>tieshape08
    STX c7shapeptr+1

; Init Objects

    LDX #MAXOBJECTS-1

; 1. Init Shots
    LDA #$C0
    STA xpos,X
    LDA #$00
    STA sprtsizetype,X
    DEX

; 2. Init void objects
    LDA #$07
InitVoids
    STA sprtsizetype,X
    DEX
    BPL InitVoids

; Init shotcountinit
    LDA waveCounter
    CMP #19
    BCC DifficultyFine
    LDA #19
DifficultyFine
    TAY
    LDA shotcounttab,Y
    STA shotcountinit
    STA shotCounter
    LDA shotspeedtab,Y
    STA shotspeedinit

OverscanLoop
    LDA INTIM
    BNE OverscanLoop

;-----------------------------------------------------------
; SCREEN MAIN LOOP START WITH VSYNC
;-----------------------------------------------------------

    VERTICAL_SYNC

;-----------------------------------------------------------
; HANDLE COUNTERS/TIMERS
;-----------------------------------------------------------

    IF COMPILE_VERSION = PAL
        LDA #$4A
    ELSE
        LDA #$2C
    ENDIF

    STA TIM64T  ; set timer for next wait

    INC SHIELDPFT+0
    INC SHIELDPFB+0
    INC SHIELDPFB+5
    LDA frameCounter
    AND #$0F
    BNE NoINC
    INC SHIELDPFT+2
    INC SHIELDPFT+3
    INC SHIELDPFB+1
    INC SHIELDPFB+3
    INC SHIELDPFB+4
NoINC

    LDA crossypos
    STA crossoffset

    INC frameCounter
    BNE .WaveTimerOk
    DEC waveTimer
    BPL .WaveTimerOk
    INC waveTimer
.WaveTimerOk
    LDA shotCounter
    BEQ .shotCounterOk
    DEC shotCounter

.shotCounterOk
    LDA expCounter
    BEQ .expCounterOk
    DEC expCounter

.expCounterOk
    JSR Random

    JSR FlickerSort

    JSR Starfield

    LDA #$45
    STA bcdScore
    LDA #$67
    STA bcdScore+1
    LDA #$89
    STA bcdScore+2

; Prepare Scoredisplay
    LDA #$0F
    STA COLUP0
    STA COLUP1
    LDA #%00011
    STA NUSIZ0
    STA NUSIZ1
    STA VDELP0
    STA VDELP1
    LDA #$00
    STA REFP1
    STA WSYNC
    LDX #7
.Wait
    DEX
    BNE .Wait
    STA.W RESP0
    STA RESP1
    LDA #$10
    STA HMP1
    LDX #11
NextDigit
    TXA
    LSR
    LSR
    TAY
    LDA bcdScore,Y      ; load low/high values of states
    STA tempVar1
    AND #$0F            ; select lower nibble
    TAY                 ; X-> shape of value
    LDA #>zero          ; Load pointer to desired value
    STA charPtr,X       ; Store sprite pointer
    STA charPtr-2,X       ; Store sprite pointer
    DEX
    LDA digittab,Y      ; Load pointer to desired value
    STA charPtr,X       ; Store sprite pointer
    DEX
    DEX
    LDA tempVar1         ; load low/high values of states
    LSR                 ; select higher nibble...
    LSR                 ; ...of status value (bcd!)
    LSR                 ; (numbers 0-9)
    LSR                 ;
    TAY                 ; X-> shape of value
    LDA digittab,Y      ; Load pointer to desired value
    STA charPtr,X       ; Store sprite pointer
    DEX
    BPL NextDigit
    LDA #$06
    STA tempVar2

Loop2
    LDY INTIM
    BNE Loop2       ;   waste time
    JMP MainScreen

;-----------------------------------------------------------
;FLICKER SORT SUB
;-----------------------------------------------------------

FlickerSort:
    LDX #MAXOBJECTS-2
SortLoop:
    LDY sprtsizetype+1,X    ; A-> 1st sprite type and size
    LDA ypos+1,X            ; A-> 1st sprite ypos
    SBC shapeheight,Y       ; A-> 1st sprite ypos - size
    SBC #$03                ; Fix skipping problem
    CMP ypos,X              ; 2nd Sprite drawable?
    BPL DontSwap            ; Y: All right!
SwapThis
    LDA xpos,X              ; N: Swap horizontal position
    LDY xpos+1,X
    STA xpos+1,X
    STY xpos,X
    LDA ypos,X                ; Swap vertical position
    LDY ypos+1,X
    STA ypos+1,X
    STY ypos,X
    LDA xymov,X             ; Swap horizontal movement
    LDY xymov+1,X
    STA xymov+1,X
    STY xymov,X
    LDA zmov,X             ; Swap horizontal movement
    LDY zmov+1,X
    STA zmov+1,X
    STY zmov,X
    LDA sprtsizetype,X             ; Swap size & type
    LDY sprtsizetype+1,X
    STA sprtsizetype+1,X
    STY sprtsizetype,X
DontSwap:
    DEX
    BPL SortLoop
AnyRTS
    RTS

;-----------------------------------------------------------
;JOYSTICK READING
;-----------------------------------------------------------

; Reading the joystick

Joy
        LDA SWCHA
        ASL
        BCS RightDone
        LDY crossxpos
        CPY #CROSSXMAX
        BEQ RightDone
        INC crossxpos

RightDone
        ASL
        BCS LeftDone
        LDY crossxpos
        CPY #CROSSXMIN
        BEQ LeftDone
        DEC crossxpos
LeftDone
        ASL
        BCS DownDone
        LDY crossypos
        BEQ DownDone
        DEC crossypos
DownDone
        ASL
        BCS UpDone
        LDY crossypos
        CPY #CROSSYMAX
        BEQ UpDone
        INC crossypos
UpDone

;-----------------------------------------------------------
;INTELLIGENT MOVE AND HANDLE ALL SPACE OBJECTS
;-----------------------------------------------------------

    LDA #$00
    STA bVoidHandled

    LDX #MAXOBJECTS-1

MoveNextSprite
    JSR GeneralMovement     ; General Movement

; Handle enemy ships
    LDA sprtsizetype,X
    CMP #$28
    BMI NoShipSprite
    DEC shipCounter         ; Silly trick pt.1...
    JSR HandleCollision     ; Handle Collision
    INC shipCounter         ; Silly trick pt.2...
    LDA #%01111110
    STA tempVar4            ; Ships are slow on z axxis
    JSR ZMovement           ; Move along the z axxis

    TYA                     ; Special ship handling
    BEQ .Spritedone2
    LDA zmov,X
    ASL
    BCS .Incoming2
    DEC shipCounter

    LDA #$07
    STA sprtsizetype,X      ; Remove Object
    BNE .Spritedone2

.Incoming2
    LDA sprtsizetype,x
    ADC #$06
    STA sprtsizetype,x      ; Special ship handling

.Spritedone2
    JMP Spritedone          ; Done

; Handle gates
NoShipSprite
    CMP #$20
    BMI NoGateSprite
    CPY #$00
    BNE .GateCollisionDone
    DEY

    TAY

.GateCollisionDone
    LDA #%00000110
    STA tempVar4            ; gates are very quick on z axxis
    JSR ZMovement           ; Move along the z axxis

    TYA                     ; Special gate handling
    BEQ .Spritedone2
    LDA #$20
    STA sprtsizetype,X      ; Special gate handling

; Handle diamonds
NoGateSprite
    CMP #$18
    BMI NodiamondSprite
    CPY #$00
    BNE .DiamondCollisionDone
    STY specialon
    JSR HandleCollision2    ; Handle Collision

.DiamondCollisionDone
    LDA #%01111110
    STA tempVar4            ; diamonds are slow on z axxis
    JSR ZMovement           ; Move along the z axxis

    TYA                     ; Special diamond handling
    BEQ .Spritedone2
    LDA #$00                ; remove a special object!
    STA specialon
    LDA #$07
    STA sprtsizetype,X      ; Special diamond handling
    BNE .Spritedone2

; Handle shots
NodiamondSprite
    CMP #$10
    BMI NoShotSprite
    JSR HandleCollision     ; Handle Collision
    LDA shotspeedinit
    STA tempVar4            ; Shots are quick on z axxis
    JSR ZMovement           ; Move along the z axxis

    LDA xpos,X              ; Special shot handling
    BMI .RemoveShot
    LDA ypos,X
    CMP #$3F
    BCS .RemoveShot
    TYA
    BEQ .Spritedone3

    LDA #$08
    STA AUDC1
    STA AUDC0

.RemoveShot
    LDA #$07
    STA sprtsizetype,X
    BNE .Spritedone3

; Handle Explosions
NoShotSprite
    CMP #$08
    BMI NoExplosionSprite
    LDA #%01111110
    STA tempVar4            ; explosions are slow on z axxis
    JSR ZMovement           ; Move along the z axxis

    TYA                     ; Special Explosion Handling
    BNE .RemoveShot
    LDA expCounter
    BEQ .RemoveShot
    AND #%00000111
    BNE .Spritedone3
    LDA sprtsizetype,X
    EOR #%00000001
    STA sprtsizetype,X      ; Special Explosion Handling
.Spritedone3
    JMP Spritedone          ; Done

; Handle void object
NoExplosionSprite
    CMP #$07
    BMI NoVoidObject
    LDA xpos,X
    ORA #%10000000
    STA xpos,X
    LDA bVoidHandled
    BMI .Spritedone3
    DEC bVoidHandled

    LDA waveTimer   ; Spawn a Warp Gate?
    BNE NoWarpSpawn
    JMP AddGate     ; Spawn a Gate!

NoWarpSpawn         ; Spawn a Ship?
    LDA waveCounter
    AND #$03
    TAY
    LDA shiptab,Y
    CMP shipCounter
    BCC NoShipSpawn
    JMP AddShip     ; Spawn a Ship!

NoShipSpawn
    LDA frameCounter    ; Spawn a Mothership?
    CMP #$40
    BNE NoMotherSpawn
    JMP AddMotherShip   ; Spawn a Mothership!

NoMotherSpawn
    CMP #$C0            ; Spawn a Diamond?
    BNE NoDiamondSpawn
    JMP AddDiamond      ; Spawn a Diamond!

NoDiamondSpawn
    LDA shotCounter     ; Spawn a Shot?
    BNE .NoSpawn
    JMP AddShot         ; Spawn a Shot!
.NoSpawn
    BNE .Spritedone3


; Handle mother ship
NoVoidObject
    CMP #$03
    BMI NoMothership

    STA tempPtr         ; Remember A

    LDA frameCounter
    AND #$03
    BNE NoSlaveI2

    LDA waveCounter     ; Move 1.5 pixels
    AND #%00000011
    BNE NoStarDestroyer2
;    INC xpos,X
NoStarDestroyer2
    CMP #$03
    BNE NoSlaveI2
    INC ypos,X
NoSlaveI2

    TYA
    BNE .Spritedone3    ; Mothership was hit?

    LDA #$01            ; Shot sound
    STA AUDC1
    LDA waveCounter     ; Begin evasive maneuvers
    AND #%00000011
    BNE NoStarDestroyer
    LDA xymov,X
    AND #%11110011
    STA xymov,X
    LDA frameCounter
    AND #%00001100
    ORA xymov,X
    STA xymov,X

NoStarDestroyer
    CMP #$03
    BNE NoSlaveI
    LDA xymov,X
    AND #%11111100
    STA xymov,X
    LDA frameCounter
    AND #%00000011
    ORA xymov,X
    STA xymov,X

NoSlaveI
    ;DEC motherCounter   ; Mothership dead?
    BPL Spritedone      ;

    LDA #$00
    STA specialon       ; Clear special!

    LDA tempPtr
    JSR HandleCollision2; Handle Collision

; Handle laser
NoMothership
    LDA frameCounter
    LSR                 ; Preset carry!
    LDA laserCounter
    BMI .CheckFire
    BCC .DontCount
    SBC #$04
    STA laserCounter
    BMI .CheckFire
.DontCount
    TAY
    LDA laserxpos,Y
    STA xpos,X
    LDA laserypos,Y
    STA ypos,X
    TYA
    LSR
    LSR
    TAY
    LDA lasertype,Y
    STA sprtsizetype,X
    JMP Spritedone

.CheckFire
    LDY #$D0
    STY xpos,X
    STY ypos,X
    LDY INPT4
    BMI Spritedone

    CLC
    ADC #$01
    AND #%00000011
    ORA #%00100000
    TAY

    STY laserCounter

Spritedone
    DEX
    BMI Handlerdone
    JMP MoveNextSprite
Handlerdone
    RTS

;-----------------------------------------------------------
;ADD NEW SHIP
;-----------------------------------------------------------

AddShip
    JSR Random
    STA zmov,x
    ORA #%10000000
    STA xpos,X
    AND #$0F
    STA xymov,x
    AND #%00001010
    BNE .XYMovementFine
    LDA xymov,x
    ORA #%00001010
    STA xymov,x

.XYMovementFine
    LDA waveCounter
    AND #$03        ;(0-3)
    TAY
    JSR Random
    AND #$7F
    STA ypos,X
    AND #$07
    ORA wavecolortbl,Y
    STA sprtsizetype,X
    INC shipCounter
    JMP Spritedone

;-----------------------------------------------------------
;ADD NEW MOTHERSHIP
;-----------------------------------------------------------

AddMotherShip
    LDA specialon
    BNE .ShipDone
    JSR Random
    ORA #%10000000
    STA specialon
    STA xpos,X
    LDA waveCounter
    AND #$03        ;(0-3)
    TAY
    JSR Random
    AND #$7F
    STA ypos,X
    LDA wavecolortbl2,Y
    STA sprtsizetype,X
    LDA mothermovetbl,Y
    STA xymov,x
    LDA #$0D            ; Make this tabled?
    ;STA motherCounter
.ShipDone
    JMP Spritedone

;-----------------------------------------------------------
;ADD NEW SHOT
;-----------------------------------------------------------

AddShot
    LDY #MAXOBJECTS/2   ; Only check half the objects...

.SearchAttacker
    LDA sprtsizetype,Y
    CMP #$28
    BMI .CheckNext
    AND #$07
    STA tempVar3
    LDA xpos,Y
    BMI .CheckNext
    LDA ypos,Y
    CMP #$3F
    BCS .CheckNext
    SBC tempVar3
    STA ypos,X
    LDA xpos,Y
    STA xpos,X
    LDA #%10000110
    STA zmov,X
    LDA #$00
    STA xymov,X
    LDA tempVar3
    LSR
    ORA #$10            ; Shotsize depends on enemy size!
    STA sprtsizetype,X
    LDA #$08            ; Shot sound
    STA AUDC1

    LDA waveCounter     ; Tie Advance behaviour!
    AND #%00000010
    BEQ SpriteMoveOk
    LDA rnd
    AND #$0F
    STA xymov,Y
    AND #%00001010
    BNE SpriteMoveOk
    LDA xymov,Y
    ORA #%00001010
    STA xymov,Y

SpriteMoveOk
    LDA shotcountinit
    STA shotCounter
    JMP Spritedone

.CheckNext
    DEY
    BPL .SearchAttacker
    JMP Spritedone

;-----------------------------------------------------------
;ITERATE A RANDOM VALUE
;-----------------------------------------------------------

Random
    LDA rnd
    BNE .skipInit
    LDA #$FF
.skipInit:
    ASL
    ASL
    ASL
    EOR rnd
    ASL
    ROL rnd
    RTS

;-----------------------------------------------------------
;HANDLE THE LASER SPRITE
;-----------------------------------------------------------

HandleLaser
    RTS

;-----------------------------------------------------------
;HANDLE GENERAL XY MOVEMENT
;-----------------------------------------------------------

GeneralMovement
    LDY xymov,X
    LDA frameCounter
    LSR
    BCC .EvenFrame
    LDY #$00

.EvenFrame
    TYA
    LSR
    AND #$01
    BCC .MoveRight
    EOR #$FF            ; Carry is set! Excellent!

.MoveRight
    ADC xpos,X          ; Carry is clear! Fine!
    STA xpos,X
    TYA
    LSR
    LSR
    LSR
    BCC .MoveUp
    EOR #$FF            ; Carry is set! Excellent!

.MoveUp
    ADC ypos,X          ; Carry is clear! Fine!
    AND #$7F
    STA ypos,X

;-----------------------------------------------------------
;COLLISION DETECTION
;-----------------------------------------------------------

    LDY sprtsizetype,X
    LDA #$46
    SBC xpos,X
    BMI .NoHit
    SBC #$10
    BPL .NoHit
    LDA #$22
    ADC shapeheight,Y
    SBC ypos,X
    BMI .NoHit
    SBC shapeheight,Y
    BPL .NoHit
    LDA laserCounter
    AND #%11111000 ; Shot expired this or next frame?
    BNE .NoHit
    LDY #$00
    RTS

.NoHit
    LDY #$FF
    RTS

;-----------------------------------------------------------
;HANDLE COLLISION
;-----------------------------------------------------------

HandleCollision
    CPY #$00
    BEQ HandleCollision2
    RTS
HandleCollision2
    TAY

    LDA #$00
    STA xymov,X         ; Clear movement

    LDY #$03
    LDA sprtsizetype,X  ; SoundFX
    CMP #$07
    BCC SpecialSFX
    LDY #$08
SpecialSFX
    STY AUDC0

    LDA #$40
    STA expCounter

    LDA sprtsizetype,X
    AND #%00000110
    ORA #%00001000
    STA sprtsizetype,X
    PLA
    PLA
    JMP Spritedone

;-----------------------------------------------------------
;ADD NEW GATE
;-----------------------------------------------------------

AddGate
    LDA specialon
    BNE .GateDone3
    JSR Random
    ORA #%10000000
    STA xpos,X          ; Gate starts out of sight
    AND #$3F
    STA ypos,X          ; Random y pos
    LDA #%10000111
    STA zmov,x          ; Init gate speed
    LDA #$00
    STA xymov,X         ; No movement
    LDA #$20
    STA sprtsizetype,x  ; Gate type
    STA specialon
    LDA #$20            ; More ferociuos attackers when gate is on
    STA shotcountinit

    LDA #4            ; Shot sound
    STA AUDC1

    JMP Spritedone
.GateDone3
    JMP NoDiamondSpawn

AddDiamond
    LDA specialon
    BNE .DiamondDone
    JSR Random
    AND #$7F
    STA xpos,X
    LDA #%10011111
    STA zmov,x
    LDA rnd
    AND #$1F
    ADC #$14
    STA ypos,X
    CMP #$20            ; 2
    LDA diamondxmov,Y    ; 4
    BCS .Movedown       ; 2
    ORA #%00001100      ; 2

.Movedown
    ORA #%00001000      ; 2
    STA xymov,X         ; 4
    LDA #$18
    STA sprtsizetype,x
    STA specialon
.DiamondDone
    JMP Spritedone

;-----------------------------------------------------------
;Z MOVEMENT
;-----------------------------------------------------------

ZMovement
    LDY #$00            ; NoOverflow
    DEC zmov,X
    LDA zmov,X
    ASL
    BNE .NoZMovement
    LDA sprtsizetype,x
    AND #%11111000
    STA tempVar3
    BCS .Incoming
    DEC sprtsizetype,x
    LDA sprtsizetype,x
    AND #$07
    CMP #$07
    BNE .MoveDone
    LDY #$FF
    JMP .MoveDone

.Incoming
    INC sprtsizetype,x
    LDA sprtsizetype,x
    AND #$07
    BNE .MoveDone
    LDY #$FF

.MoveDone
    ORA tempVar3
    STA sprtsizetype,x
    LDA zmov,X
    ORA tempVar4      ; 11111110 slow - 00011110 fast
    STA zmov,X

.NoZMovement
    RTS

Starfield
    LDA #$30
    STA yposBackup
    LDA #$20
    STA yposBackup+1
    LDA #41
    STA yposBackup+2

    LDA #$50
    LDX #$02
    JSR PosElement
    INX
    LDA #$40
    JSR PosElement
    INX

    LDY #$07
    LDA frameCounter
    LSR
    BCS LeftCannon
    LDY #$A0
LeftCannon
    TYA

    JSR PosElement
    LDX #$00
    LDA crossxpos
    JSR PosElement
    STA WSYNC
    STA HMOVE
    STA WSYNC
    STA HMCLR
    RTS

PosElement
        SEC
        STA WSYNC                ;
.waitzone
        SBC #$0F                 ;
        BCS .waitzone            ; RESP loop
        EOR #$07                 ;
        ASL                      ;
        ASL                      ;
        ASL                      ;
        ASL                      ;
        STA HMP0,X
        STA RESP0,X
        RTS                      ; done, that's all!

DrawScore
; slightly modified 48 pixel version:
loop2
        LDY tempVar2           ;+3  63  189
        SLEEP 4
        LDA     (charPtr),y     ;+5  68  204
        STA     WSYNC           ;go
        STA.w   GRP0            ;+3  71  213      D1     --      --     --
        LDA     (charPtr+$2),y  ;+5   5   15
        STA     GRP1            ;+3   8   24      D1     D1      D2     --
        LDA     (charPtr+$4),y  ;+5  13   39
        STA     GRP0            ;+3  16   48      D3     D1      D2     D2
        LDA     (charPtr+$6),y  ;+5  21   63
        STA     tempVar1         ;+3  24   72
        LAX     (charPtr+$8),y  ;+5  29   87
        LDA     (charPtr+$A),y  ;+5  36  108
        LDY     tempVar1         ;+3  41  123              !
        STY     GRP1            ;+3  44  132      D3     D3      D4     D2!
        STX     GRP0            ;+3  47  141      D5     D3!     D4     D4
        STA     GRP1            ;+3  50  150      D5     D5      D6     D4!
        STA     GRP0            ;+3  53  159      D4*    D5!     D6     D6
        DEC     tempVar2           ;+5  58  174                             !
        BPL     loop2           ;+2  60  180
        LDY #0
        STY GRP0
        STY GRP1
        STY GRP0
        STY NUSIZ0
        STY NUSIZ1
        STY VDELP0
        STY VDELP1
        RTS                      ; done, that's all!

ROCKETCOL
                                .byte   BLACK
                                .byte   RED
                                .byte   RED
                                .byte   BLUE
                                .byte   BLUE
                                .byte   BLUE
                                .byte   BLUE
                                .byte   WHITE
                                .byte   WHITE
                                .byte   WHITE
                                .byte   WHITE
                                .byte   WHITE
                                .byte   WHITE
                                .byte   BLUE
                                .byte   BLUE
                                .byte   BLUE

TOWERSCOL
                                .byte   BLACK
                                .byte   GREY
                                .byte   GREY
                                .byte   BLUE
                                .byte   BLUE
                                .byte   GREEN
                                .byte   GREEN
                                .byte   GREY
                                .byte   GREY
                                .byte   GREY
                                .byte   GREY
                                .byte   BLUE
                                .byte   BLUE
                                .byte   GREEN
                                .byte   GREEN
                                .byte   WHITE

DOMECOL
                                .byte   BLACK
                                .byte   BLACK
                                .byte   BLACK
                                .byte   BLUE
                                .byte   BLUE
                                .byte   BLUE
                                .byte   BLUE
                                .byte   BLUE
                                .byte   BLUE
                                .byte   RED
                                .byte   YELLOW
                                .byte   RED
                                .byte   BLUE
                                .byte   BLUE
                                .byte   BLUE
                                .byte   BLUE

SMARTIECOL
                                .byte   BLACK
                                .byte   GREEN
                                .byte   BLUE
                                .byte   GREEN
                                .byte   BLUE
                                .byte   GREY
                                .byte   GREY
                                .byte   BLACK

BASECOL
                                .byte   BLUE
                                .byte   BLUE
                                .byte   BLUE
                                .byte   BLUE
                                .byte   BLUE
                                .byte   GREEN
                                .byte   GREEN
                                .byte   WHITE

CANNONCOL
                                .byte   BROWN
                                .byte   BROWN
                                .byte   RED
                                .byte   RED
                                .byte   RED
                                .byte   RED
                                .byte   RED
                                .byte   RED
                                .byte   RED
                                .byte   RED

crosshair
    ds CROSSPACE, 0
    .byte %00010000
    .byte %00010000
    .byte %00000000
    .byte %11000110
    .byte %00000000
    .byte %00010000
    .byte %00010000
    ds CROSSPACE, 0

;-----------------------------------------------------------
;DATA BLOCK
;-----------------------------------------------------------

    ORG     $FA00
shapeheight
    .byte $03,$03,$03,$03,$03,$03,$03,$03
    .byte $03,$03,$03,$03,$03,$03,$03,$03
    .byte $03,$03,$03,$03,$03,$03,$03,$03
    .byte $03,$03,$03,$03,$03,$03,$03,$03
    .byte $03,$03,$03,$03,$03,$03,$03,$03
    .byte $03,$03,$03,$03,$03,$03,$03,$03
    .byte $03,$03,$03,$03,$03,$03,$03,$03

shapeclrtab
    IF COMPILE_VERSION = PAL
        .byte $B8,$BA,$BC,$4F,$5F,$DF,$0F,$0F
        .byte $60,$2E,$40,$2E,$60,$2E,$40,$2E
        .byte $60,$60,$62,$62,$64,$64,$66,$66
        .byte $0F,$8E,$0F,$8E,$0F,$8E,$0F,$8E
        .byte $0F,$8E,$0F,$8E,$0F,$8E,$0F,$8E
        .byte $50,$50,$52,$52,$54,$54,$56,$56
        .byte $D0,$D0,$D2,$D2,$D4,$D4,$D6,$D6
    ELSE
        .byte $98,$9A,$9C,$3F,$CF,$8F,$0F,$0F
        .byte $40,$2E,$30,$2E,$34,$2E,$40,$2E
        .byte $40,$40,$42,$42,$44,$44,$46,$46
        .byte $0F,$5E,$0F,$5E,$0F,$5E,$0F,$5E
        .byte $0F,$5E,$0F,$5E,$0F,$5E,$0F,$5E
        .byte $C0,$C0,$C2,$C2,$C4,$C4,$C6,$C6
        .byte $80,$80,$82,$82,$84,$84,$86,$86
    ENDIF

shapeptrtab
    .byte #<laser03,#<laser02,#<laser01,#<mothership01,#<mothership03,00,00,00
    .byte #<explosion02,#<explosion02,#<explosion04,#<explosion04,#<explosion06,#<explosion06,#<explosion08,#<explosion08
    .byte #<shot01,#<shot02,#<shot03,#<shot04,#<shot05,#<shot06,#<shot07,#<shot08
    .byte #<diamond01,#<diamond01,#<diamond01,#<diamond01,#<diamond01,#<diamond01,#<diamond01,#<diamond01
    .byte #<gate01,#<gate02,#<gate01,#<gate02,#<gate01,#<gate02,#<gate01,#<gate02
    .byte #<tieshape01,#<tieshape02,#<tieshape03,#<tieshape04,#<tieshape05,#<tieshape06,#<tieshape07,#<tieshape08
    .byte #<tieshape01,#<tieshape02,#<tieshape03,#<tieshape04,#<tieshape05,#<tieshape06,#<tieshape07,#<tieshape08

digittab
    .byte #<zero,#<one,#<two,#<three,#<four
    .byte #<five,#<six,#<seven,#<eight,#<nine

zero
    .byte %00111100
    .byte %01000010
    .byte %01000010
    .byte %01000010
    .byte %01000010
    .byte %01000010
    .byte %00111100

one
    .byte %00001000
    .byte %00001000
    .byte %00001000
    .byte %00001000
    .byte %00001000
    .byte %00011000
    .byte %00001000

two
    .byte %01111110
    .byte %01000000
    .byte %00100000
    .byte %00011100
    .byte %00000010
    .byte %01000010
    .byte %00111100

three
    .byte %00111100
    .byte %01000010
    .byte %00000010
    .byte %00111100
    .byte %00000010
    .byte %01000010
    .byte %00111100

four
    .byte %00000100
    .byte %00000100
    .byte %01111110
    .byte %00100100
    .byte %00010100
    .byte %00001100
    .byte %00000100

five
    .byte %00111100
    .byte %01000010
    .byte %00000010
    .byte %01111100
    .byte %01000000
    .byte %01000000
    .byte %01111100

six
    .byte %00111100
    .byte %01000010
    .byte %01000010
    .byte %01111100
    .byte %01000000
    .byte %01000010
    .byte %00111100

seven
    .byte %01000000
    .byte %00100000
    .byte %00010000
    .byte %00001000
    .byte %00000100
    .byte %00000010
    .byte %01111110

eight
    .byte %00111100
    .byte %01000010
    .byte %01000010
    .byte %00111100
    .byte %01000010
    .byte %01000010
    .byte %00111100

nine
    .byte %00111100
    .byte %01000010
    .byte %00000010
    .byte %00111110
    .byte %01000010
    .byte %01000010
    .byte %00111100

lasertype
    .byte $02,$02,$02,$01,$01,$01,$00,$00,$00

; Page with Space objects
    ORG     $FC00

shot04
laser02
    .byte %00111100
    .byte %01000010
    .byte %00111100
shot03
laser01
    .byte %00111100
    .byte %01000010
    .byte %00111100
shot02
    .byte %00111100
    .byte %01000010
    .byte %00111100
shot01
tieshape01
advshape01
    .byte %00111100
    .byte %01000010
    .byte %00111100

shot08:
    .byte %00111100
    .byte %01000010
    .byte %00111100

shot07:
    .byte %00111100
    .byte %01000010
    .byte %00111100

shot06:
    .byte %00111100
    .byte %01000010
    .byte %00111100

shot05:
    .byte %00111100
    .byte %01000010
    .byte %00111100

laser03:
    .byte %00111100
    .byte %01000010
    .byte %00111100

explosion08
    .byte %00111100
    .byte %01000010
    .byte %00111100

explosion06
    .byte %00111100
    .byte %01000010
    .byte %00111100

explosion04
    .byte %00111100
    .byte %01000010
    .byte %00111100

explosion02
    .byte %00111100
    .byte %01000010
    .byte %00111100

explosion208
    .byte %00111100
    .byte %01000010
    .byte %00111100

explosion206
    .byte %00111100
    .byte %01000010
    .byte %00111100

explosion204
    .byte %00111100
    .byte %01000010
    .byte %00111100

explosion202
    .byte %00111100
    .byte %01000010
    .byte %00111100

diamond01
    .byte %00111100
    .byte %01000010
    .byte %00111100

diamond02
    .byte %00111100
    .byte %01000010
    .byte %00111100

gate01
    .byte %00111100
    .byte %01000010
    .byte %00111100

gate02
    .byte %00111100
    .byte %01000010
    .byte %00111100

mothership01:
    .byte %00111100
    .byte %01000010
    .byte %00111100

mothership02:
    .byte %00111100
    .byte %01000010
    .byte %00111100

mothership03:
    .byte %00111100
    .byte %01000010
    .byte %00111100

mothership04:
    .byte %00111100
    .byte %01000010
    .byte %00111100

tieshape08
    .byte %00111100
    .byte %01000010
    .byte %00111100

tieshape07
    .byte %00111100
    .byte %01000010
    .byte %00111100

tieshape06
    .byte %00111100
    .byte %01000010
    .byte %00111100

tieshape05
    .byte %00111100
    .byte %01000010
    .byte %00111100

tieshape04
    .byte %00111100
    .byte %01000010
    .byte %00111100

tieshape03
    .byte %00111100
    .byte %01000010
    .byte %00111100

tieshape02
    .byte %00111100
    .byte %01000010
    .byte %00111100

diamondxmov
    .byte %00000011,%00000011,%00000011,%00000011,%00000011,%00000011,%00000011,%00000001
    .byte %00000000,%00000010,%00000010,%00000010,%00000010,%00000010,%00000010,%00000010

laserxpos
    .byte $3E,$3E,$3E,$3E
    .byte $38,$44,$44,$38
    .byte $32,$4A,$4A,$32
    .byte $2C,$50,$50,$2C
    .byte $26,$56,$56,$26
    .byte $20,$5C,$5C,$20
    .byte $1A,$62,$62,$1A
    .byte $14,$68,$68,$14
    .byte $0E,$6E,$6E,$0E

hmovetab
    .byte $80,$70,$60,$50,$40,$30,$20,$10,$00
    .byte $F0,$E0,$D0,$C0,$B0,$A0,$90

laserypos
    .byte $23,$23,$23,$23
    .byte $26,$26,$20,$20
    .byte $29,$29,$1A,$1A
    .byte $2C,$2C,$17,$17
    .byte $2F,$2F,$14,$14
    .byte $32,$32,$11,$11
    .byte $35,$35,$0E,$0E
    .byte $38,$38,$0B,$0B
    .byte $3B,$3B,$08,$08

wavecolortbl
    .byte %00101000,%00110000,%00111000,%01000000

wavecolortbl2
    .byte $03,$04,$05,$06

mothermovetbl
    .byte %00000010,%00000000,%00000000,%00001000

    ORG $FF00

BOTTOMPF0
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %01110000
                                .byte   %01110000
                                .byte   %01110000
                                .byte   %01110000
                                .byte   %01110000
                                .byte   %01110000
                                .byte   %01110000
                                .byte   %01110000
                                .byte   %01110000
                                .byte   %01110000
                                .byte   %01110000
                                .byte   %01110000
                                .byte   %01110000
                                .byte   %01110000
                                .byte   %01110000
                                .byte   %01110000
                                .byte   %01110000
                                .byte   %01110000
                                .byte   %01110000
                                .byte   %01110000
                                .byte   %01110000
                                .byte   %01110000
                                .byte   %00110000
                                .byte   %00110000
                                .byte   %00110000
                                .byte   %00110000
                                .byte   %00110000
                                .byte   %00110000
                                .byte   %00110000
                                .byte   %00110000
                                .byte   %00110000
                                .byte   %00110000
                                .byte   %00110000
                                .byte   %00110000
BOTTOMPF1
                                .byte   %11111111
                                .byte   %11111111
                                .byte   %00111111
                                .byte   %00111111
                                .byte   %00111100
                                .byte   %00111100
                                .byte   %00011000
                                .byte   %00011000
                                .byte   %00011000
                                .byte   %00011000
                                .byte   %00011000
                                .byte   %00011000
                                .byte   %00001000
                                .byte   %00001000
                                .byte   %00001000
                                .byte   %00001000
                                .byte   %00001000
                                .byte   %00001000
                                .byte   %00001000
                                .byte   %00001000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000

BOTTOMPF2
                                .byte   %11111111
                                .byte   %11111111
                                .byte   %11111111
                                .byte   %11111111
                                .byte   %11111100
                                .byte   %11111100
                                .byte   %11111100
                                .byte   %11111100
                                .byte   %11111100
                                .byte   %11111000
                                .byte   %11111000
                                .byte   %11111000
                                .byte   %11111000
                                .byte   %11111000
                                .byte   %11111000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11000000
                                .byte   %11000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000

ROCKET
                                .byte   %00000000
                                .byte   %01111111
                                .byte   %01111111
                                .byte   %01111111
                                .byte   %01111111
                                .byte   %00111110
                                .byte   %00111110
                                .byte   %00011100
                                .byte   %00011100
                                .byte   %00010100
                                .byte   %00011100
                                .byte   %00010100
                                .byte   %00011100
                                .byte   %00011100
                                .byte   %00011100
                                .byte   %00001000

SMARTIEL
                                .byte   %00000000
                                .byte   %10100000
                                .byte   %01010000
                                .byte   %10100000
                                .byte   %01010000
                                .byte   %11110000
                                .byte   %01100000
                                .byte   %00000000

SMARTIER
                                .byte   %00000000
                                .byte   %00001010
                                .byte   %00000101
                                .byte   %00001010
                                .byte   %00000101
                                .byte   %00001111
                                .byte   %00000110
                                .byte   %00000000

TOWERS
                                .byte   %00000000
                                .byte   %00111000
                                .byte   %00111000
                                .byte   %11111110
                                .byte   %11111110
                                .byte   %11111110
                                .byte   %01111100
                                .byte   %00111000
                                .byte   %00111000
                                .byte   %00111000
                                .byte   %01111100
                                .byte   %11111110
                                .byte   %11111110
                                .byte   %11111110
                                .byte   %01111100
                                .byte   %00111000

BASE
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %11111110
                                .byte   %11111110
                                .byte   %11111110
                                .byte   %01111100
                                .byte   %00111000


DOME
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %01111110
                                .byte   %00011000
                                .byte   %00011000
                                .byte   %00111100
                                .byte   %00111100
                                .byte   %01111110
                                .byte   %11111111
                                .byte   %11011011
                                .byte   %11111111
                                .byte   %01111110
                                .byte   %00011000
                                .byte   %00011000
                                .byte   %00000000

CANON
    .byte   %11111111
    .byte   %11111111
    .byte   %00100000
    .byte   %00100000
    .byte   %00111100
    .byte   %00111100
    .byte   %00100000
    .byte   %00100000
    .byte   %00100000
    .byte   %00100000

shiptab
    .byte 4
    .byte 4
    .byte 4
    .byte 4

shotcounttab
    .byte 140, 100, 80, 68, 64, 60, 56, 52, 48, 44
    .byte 40, 38, 36, 34, 32, 30, 28, 26, 24, 22

shotspeedtab
    .byte 25, 23, 21, 19, 17, 15, 13, 11, 9, 7
    .byte 7,7,7,7,7,7,7,7,7,7

    ORG $FFFA
    .word Start, Start, Start
